export declare type F<T> = (...args: any[]) => T;
export declare class Transaction {
    private static id;
    readonly id: number;
    readonly hint: string;
    readonly snapshot: number;
    readonly error: any;
    private _snapshot;
    private changeset;
    private pending;
    private sealed;
    private _error;
    private resultPromise;
    private resultResolve;
    private resultReject;
    private conflicts;
    private refreshments;
    constructor(hint?: string);
    static readonly current: Transaction | undefined;
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    commit(): void;
    sealToCommit(): Transaction;
    discard(error?: any): Transaction;
    waitForFinish(): Promise<void>;
    finished(): boolean;
    undo(): void;
    redo(): void;
    static run<T>(hint: string, func: F<T>, ...args: any[]): T;
    static runAsync<T>(hint: string, func: F<Promise<T>>, ...args: any[]): Promise<T>;
    private _run;
    private sealAndRethrowErrorIfNeeded;
    private static seal;
    private checkConflictsResolved;
    private tryResolveConflicts;
    static _acquireSnapshotNumber(t: Transaction): number;
    private performCommit;
    private performDiscard;
    private acquirePromise;
    static _changeset(t: Transaction): Map<Handle, Record>;
    static _wrap<T>(t: Transaction, c: Cache | undefined, inc: boolean, dec: boolean, func: F<T>): F<T>;
}
export declare class StoreParams {
    readonly name?: string;
    readonly idProperty?: string;
    readonly isObjectWithId?: (obj: object) => boolean;
    readonly twoWaySync?: (incomingPatches: object[], outgoingPatches: object[]) => void;
}
export declare class Store {
    readonly params: StoreParams;
    constructor(params?: StoreParams);
    static dispose(obj: object): void;
}
declare class Handle {
    private static id;
    readonly id: number;
    readonly store: Store;
    readonly proxy: any;
    private committed;
    private editable;
    private editors;
    constructor(store: Store, proxy: any);
    getCommitted(): Record;
    getRelevant(t: Transaction, c: Cache | undefined): Record | undefined;
    getForView(): Record;
    getForEdit(prop: PropertyKey, value: any): Record | undefined;
    getForRenew(prop: PropertyKey): Record;
    static applyChangeset(commit: boolean, changeset: Map<Handle, Record>): void;
    dispose(): void;
    readonly hint: string;
}
declare class Record {
    readonly handle: Handle;
    readonly previous: Record;
    readonly tran: Transaction;
    readonly data: any;
    readonly changes: Set<PropertyKey>;
    readonly conflicts: Map<PropertyKey, Record>;
    readonly observers: Map<PropertyKey, Set<Cache>>;
    readonly obsolete: Set<PropertyKey>;
    constructor(handle: Handle, previous: Record | undefined, tran: Transaction, data: object);
    readonly isGenesis: boolean;
    readonly isDisposal: boolean;
    freeze(): void;
    registerPropView(prop: PropertyKey, cache: Cache | undefined): void;
    registerPropEdit(prop: PropertyKey, changed: boolean): void;
    determineConflicts(): boolean;
    markPropObsolete(prop: PropertyKey): void;
    propagateChanges(refreshments: Cache[]): number;
    static markAllObsolete(cause: string, observers: Set<Cache> | undefined, refreshments?: Cache[]): number;
    hint(prop?: PropertyKey): string;
}
declare class Member {
    readonly isTran: boolean;
    readonly isAsync: boolean;
    readonly name: PropertyKey;
    readonly body: (...args: any[]) => any;
    constructor(isTran: boolean, isAsync: boolean, name: PropertyKey, body: F<any>);
}
declare class Cache {
    readonly record: Record;
    readonly member: Member;
    readonly trap: (...args: any[]) => any;
    readonly observables: Map<PropertyKey, Set<Record>>;
    args: any[];
    value: any;
    dirty: string | undefined;
    obsolete: boolean;
    error: any;
    latency: number;
    readonly margin: number;
    constructor(owner: Record, member: Member, latency: number, margin: number);
    run<T>(func: F<T>, ...args: any[]): T;
    wrap<T>(func: F<T>): F<T>;
    markObsolete(cause: string, refreshments?: Cache[]): number;
    markDirty(cause: string, view: boolean): void;
    refresh(now: boolean): void;
    private createTrap;
    private renew;
    private renewAsync;
    readonly hint: string;
}
export interface Status {
    value: any;
    obsolete: boolean;
    error: any;
    latency: number;
}
export declare function statusof(method: F<any>): Status;
export declare function autorenew(latency: number, method: F<any>, ...args: any[]): void;
export declare function dispose(obj: object | undefined): void;
export declare const transactional: typeof tran;
export declare function tran(target: object, prop?: PropertyKey, descriptor?: TypedPropertyDescriptor<F<any>>): any;
export declare function cache(ctor: object, prop: PropertyKey, descriptor: TypedPropertyDescriptor<F<any>>): any;
export {};
